/*---------------------------------------------------------------------------------
 *	pidControl.cpp
 *
 *	Author:		Paul Alting van Geusau
 *	Created	: 	23 Sep 2013
 *	Last Mod: 	07 Dec 2013
 *	Inspired from the following code:
 *		https://github.com/murmlgrmpf/PID
 *		https://github.com/br3ttb/Arduino-PID-Library
 *		http://brettbeauregard.com/blog/category/pid/
 */
#include	"pidControl.h"

PID::PID(int16_t _sp, int16_t _kp, int16_t _ki, int16_t _kd, int16_t _outMin, int16_t _outMax) {
	spPID = _sp;
	kpPID = _kp;
	kiPID = _ki;
	kdPID = _kd;
	outMinPID = _outMin;
	outMaxPID = _outMax;

	intPID	= 0;
	errPID	= 0;
	derPID	= 0;
	pvLast	= 0;
	cvPID	= 0;
	pvPID	= 0;
}

/*---------------------------------------------------------------------------------
 *	PID control block
 */
void PID::loop() {
	errPID = pvPID - spPID;													// calculate setpoint error:
	intPID = intPID + kiPID * errPID;										// calculate integral term:
	if (intPID > outMaxPID) {												// clamp integral to paramtized maximum:
		intPID = outMaxPID;
	}
	else if (intPID < outMinPID) {											// clamp integral to paramtized minimum:
		intPID = outMinPID;
	}

	derPID = pvPID - pvLast;												// calculate derivative term:

	cvPID = kpPID * errPID + intPID - kdPID * derPID;						// calculate pid output:
	if (cvPID <= outMinPID) {												// clamp pid output value to 0 - 1023 for pwm:
		cvPID = outMinPID;													// clamp output to paramtized minimum, typically 0:
	}
	else if (cvPID > outMaxPID) {											// clamp output to paramtized maximum, typically 1023:
		cvPID = outMaxPID;
	}

	pvLast = pvPID;															// remember input value for next time:
}

void PID::spChange(int16_t _sp) {
	if ((_sp >= outMinPID) && (_sp <= outMaxPID)) {
		spPID = _sp;
	}
}

void PID::kpChange(int16_t _kp) {
	if ((_kp >= 0) && (_kp <= 100)) {
		kpPID = _kp;
	}
}
